#include "../src/object_recognition.cpp"
#include "object_recognition_node_header.hpp"

using namespace cv;
using namespace std;

bool show=false; //used as a control var for displaying image from IMAGE_TOPIC
bool analyze=false; //used as a control var for activating object recognition in the given scene.
bool write_image=false; //used as a control var for writing the image "write_image.jpg".
/* analyze_object_recognition function
 * This function sends the result corners of the object recognition to RESULT_TOPIC. 
 * If it managed to recognize the object within the scene, it publishes the corner points array of the object.
 * If no useful recognition was made, send an array of size 8 of 0's.
 */
void analyze_object_recognition()
{
  //object_recognition function initializes the result we want to return.
  object_recognition();      
  if(result.corners[0].x == 0 && result.corners[0].y == 0 &&
    result.corners[1].x == 0 && result.corners[0].y == 0 &&
    result.corners[2].x == 0 && result.corners[0].y == 0 &&
    result.corners[3].x == 0 &&result.corners[0].y == 0 &&
    sample_count<MAX_SAMPLE_COUNT)
  {
    //This means that the result was not good so we want to conitnue analyzing until we achieve a good result.
    analyze = true;
  }
  else
  {
    //we found a good result so we no longer want to analyze anything.
    analyze = false; 
  }
  //Publishing:
  if(analyze==false || sample_count==MAX_SAMPLE_COUNT)
  {
    if(ros::ok())
    {
      ros::NodeHandle nh;
      ros::Publisher pub = nh.advertise<std_msgs::Float32MultiArray>(RESULT_TOPIC, 100);
      std_msgs::Float32MultiArray result_array;
      result_array.data.clear();
      for (int i = 0; i < 4; i++) 
      {
	  printf("Result Corner to be published p[%d]=(%f,%f)\n", i, result.corners[i].x, result.corners[i].y);
	  result_array.data.push_back(result.corners[i].x);
	  result_array.data.push_back(result.corners[i].y);
      }
      for(int j=0;j<250;j++)
      {
	
	//Result_array includes all 4 corners, if the result was not a well enough, result.corners will be an array of 8 X 0's
	//After the for loop, the result_array holds the corners we want to publish. 
	//Publish array
	pub.publish(result_array);
	ros::Duration(0.02);
      }
    }
  }
}
/* read_image_callback function
 * This callback function deals with:
 * show 
 * write image
 * analyze
 */
void read_image_callback(const sensor_msgs::ImageConstPtr& msg)
{
  //Show opens up windows that views the image seen from the camera
  if(show)
  {
    try
    {
      printf("Showing Image\n");
      namedWindow("Received Image");
      startWindowThread();
      cv::imshow("Received Image", cv_bridge::toCvShare(msg, "bgr8")->image);
      waitKey(DELAY);
      destroyWindow("Received Image");
    }
    catch (cv_bridge::Exception& e)
    {
      ROS_ERROR("Could not convert from '%s' to 'bgr8'.", msg->encoding.c_str());
    }
  }
  //Write an image from IMAGE_TOPIC to the file "output image.jpg"
  if(write_image)
  {
    try
    {
      //Constructing a Mat from the given image in msg.
      
      cv::Mat scene(cv_bridge::toCvShare(msg, "bgr8")->image);
      vector<int> compression_params;
      compression_params.push_back(CV_IMWRITE_JPEG_QUALITY);
      compression_params.push_back(95);
      //namedWindow("Received Image");
      //startWindowThread();
      //cv::imshow("Received Image", scene);
      //waitKey(DELAY);
      bool ans = imwrite(WRITE_PATH , scene, compression_params);
      if(ans)
      {
	printf("Image written\n");
      }
      else
      {
	printf("Failed to write image\n");
      }
      write_image=false;
    }
    catch (cv_bridge::Exception& e)
    {
      ROS_ERROR("Could not convert from '%s' to 'bgr8'.", msg->encoding.c_str());
    }
  }
  /* Analyze sents the result of the object recognition to RESULT_TOPIC. 
   * If it managed to recognize the object within the scene, it publishes the corner points array of the object.
   * If no useful recognition was made, send an array of size 8 of 0's.
   */
  if(analyze)
  {
    //Constructing a Mat from the given image in msg.
    try
    {
      printf("Reached Analyze\n");
      cv::Mat scene_temp(cv_bridge::toCvShare(msg, "bgr8")->image);
      scene = scene_temp;
      analyze_object_recognition();
      sample_count++;
    }
    catch (cv_bridge::Exception& e)
    {
      ROS_ERROR("Could not convert from '%s' to 'bgr8'.", msg->encoding.c_str());
    }
  }
}
/* activate_control_callback function
 * This callback function is used as a control over displaying images from the camera and object recognition request.
 * This callback function gets a control std_msgs::Int32 with a value between 0 to 3.
 * 0 specifies reset (makes show,write_image,analyze=false)
 * 1 specifies showing the image from IMAGE_TOPIC
 * 2 specifies writing an image from IMAGE_TOPIC to a file
 * 3 specifies activating object recognition on the given scene from IMAGE_TOPIC.
 */
void activate_control_callback(const std_msgs::Int32::ConstPtr& msg)
{
  int control=msg->data;
  if(control==0)
  {
    show=false;
    analyze=false;
    write_image=false;
    sample_count=0;
    printf("Got Reset\n");
  }
  else if(control==1)
  {
    show=true;
    printf("Got Show\n");
  }
  else if(control==2)
  {
    write_image=true;
    printf("Got Write\n");
  }
  else if(control==3)
  {
    if(!analyze)
    {
      analyze=true;
      sample_count=0;
      printf("Got Analyze\n");
    }
  }
}
//initializes the templates - You may want to fill this in yourself.
void init_templates()
{/*
  templates[0] =  imread("src/object_recognition/templates/template_0.jpg", CV_LOAD_IMAGE_COLOR);
  templates[1] =  imread("src/object_recognition/templates/template_1.jpg", CV_LOAD_IMAGE_COLOR);
  templates[2] =  imread("src/object_recognition/templates/template_2.jpg", CV_LOAD_IMAGE_COLOR);
  templates[3] =  imread("src/object_recognition/templates/template_3.jpg", CV_LOAD_IMAGE_COLOR);
  templates[4] =  imread("src/object_recognition/templates/template_4.jpg", CV_LOAD_IMAGE_COLOR);
  templates[5] =  imread("src/object_recognition/templates/template_5.jpg", CV_LOAD_IMAGE_COLOR);
  templates[6] =  imread("src/object_recognition/templates/template_6.jpg", CV_LOAD_IMAGE_COLOR);
  templates[7] =  imread("src/object_recognition/templates/template_7.jpg", CV_LOAD_IMAGE_COLOR);
  templates[8] =  imread("src/object_recognition/templates/template_8.jpg", CV_LOAD_IMAGE_COLOR);*/
  templates[0] =  imread("src/object_recognition/templates/t0.jpg", CV_LOAD_IMAGE_COLOR);
  templates[1] =  imread("src/object_recognition/templates/t1.jpg", CV_LOAD_IMAGE_COLOR);
}

/* Node information
 * Subscribed to IMAGE_TOPIC, CONTROL_TOPIC
 * Publishing to RESULT_TOPIC (Only from analyze_object_recognition)
 */
int main(int argc, char **argv)
{
  ros::init(argc, argv, NODE_NAME);
  init_templates();
  ros::NodeHandle n;
  image_transport::ImageTransport it(n);
  ros::Subscriber control = n.subscribe(CONTROL_TOPIC, 10, activate_control_callback);
  image_transport::Subscriber image_listener = it.subscribe(IMAGE_TOPIC, 10, read_image_callback);
  ros::spin();
  return 0;
}